#!/usr/bin/perl

#
# (c) Copyright 2004 Software Garden, Inc.
# All Rights Reserved.
# Subject to Software License at the end of this file
#

#
# HBS'79 Prize Drawing Program
#
# This is the main software for running the program.
# It is called by hbs79.pl and hbs79win.pl.
#
# Use the interface program appropriate to your needs.
#

   use strict;
   use CGI qw(:standard);
   use Time::Local;
   use utf8;

#
# Export symbols
#

package hbs79;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw($programname %config_values process_request);
our $VERSION = '1.0';

# # # # # # # # # #
#
# HTML Constants & Templates
#
# # # # # # # # # #

our $programname = "HBS79 Prize Drawing Program 1.0";

my @tablist = qw(Setup Test Items Drawing Quit);

my $STYLE_STRING = <<"EOF";

body {
  background-color:#CCFFFF;
}

body, td, input, textarea, select {
  font-size:small;
  font-family:verdana,helvetica,sans-serif;
}
td {
  vertical-align: top;
}
form {
  margin:0px;
  padding:0px;
}
.sectiondark {
  border: 1px solid #99CC99;
  padding: 8px;
}
.sectionplain {
  padding: 10px;
}
.title {
  color: #006600;
  font-weight:bold;
  margin: 0em 0px 2pt 0px;
  padding: 0px 0px 0px 0px;
}
.title2 {
  border-top: 1px solid #006600;
  color: #006600;
  font-weight:bold;
  margin: .5em 0px 0px 0px;
  padding: 0px 0px 0px 0px;
}
.pagetitle {
  color: #006600;
  font-weight:bold;
  margin: 0em 0px 0px 0px;
  padding: 0px 0px 0px 0px;
}
.pagefeedinfo {
  color: black;
  font-size: smaller;
  margin: 1pt 0px 2pt 0px;
  padding: 0px 0px 0px 0px;
}
.desc {
  font-size:smaller;
  padding: 0px 0px .75em 0px;
}
.tabtable {font-size:small;}
.tab {
  border-bottom: 1px solid black;
  padding-bottom: 4px;
  }
.tab input {
  background-color:#CCCC99;
  color:black;
}
.tabselected {
  border-left: 1px solid black;
  border-right: 1px solid black;
  border-top: 1px solid black;
  background-color:#DDFFDD;
  color:black;
  padding: 1px 14px 0px 14px;
  text-align: center;
  font-weight: bold;
  }
.tab1 {
  border-bottom: 1px solid black;
  }
.tab2 {
  background-color:#DDFFDD;
  }
.tab2left {
  border-left: 1px solid black;
  background-color:#DDFFDD;
  padding-left:20px;
  }
.tab2right {
  border-right: 1px solid black;
  background-color:#DDFFDD;
  padding-left:20px;
  }
.ttbody {
  background-color:#DDFFDD;
  border-right: 1px solid black;
  border-left: 1px solid black;
  border-bottom: 1px solid black;
  padding: 0px 10px 4px 10px;
}
.head {
  border: 1px dashed #006600;
  padding: 6px;
  color: #006600;
  font-weight: bold;
  margin: 1em 0px .5em 0px;
}
.footer {
  border: 1px dashed #006600;
  padding: 6px;
  color: #006600;
  font-size: smaller;
  margin: 1em 0px .5em 0px;
}
input.small {
  font-size: smaller;
}
.itemheader {
  color: #006600;
  font-weight:bold;
  margin: .5em 0px 0px 0px;
  padding: 0px 0px 0px 0px;
}
.itemselected {
  color: #990000;
  font-weight:bold;
  margin: .5em 0px 0px 0px;
  padding: 0px 0px 0px 0px;
}
.selectedexample {
  color: #990000;
}
.itemtitle {
  border-left: 1px solid #99CC99;
  padding-left: 8px;
  margin: 4px 0px 0px 24px;
  font-weight: bold;
}
.itemlink {
  border-left: 1px solid #99CC99;
  padding: 0px 0px 0px 8px;
  margin-left: 24px;
  font-size: smaller;
}
.itemdesc {
  border-left: 1px dotted #99CC99;
  padding: 8px 0px 8px 8px;
  margin-left: 24px;
  font-size: smaller;
}
.itemmisc {
  border-left: 1px solid #99CC99;
  padding: 8px 0px 8px 8px;
  margin-left: 24px;
  font-size: smaller;
}
.itembuttons {
  float:right;
  clear:right;
  padding:5px 0px 0px 30px;
  border-top:1px solid #99CC99;
  margin:.5em 0px 0px 0px;
}
.warning {
  color: red;
}
.smallprompt {
  font-size: smaller;
}
.tdsmall {
  font-size: xx-small;
}
.itemmisc td {
  font-size: xx-small;
}
.itemdraw td {
  font-size: large;
}
td.centered {
  vertical-align: middle;
}  
EOF

   my $datafiledefault = "drawingdata.txt"; # default filename prefix to hold data and settings
   my $socketdefault = 6555; # default socket to listen to for browser

#
# Define configuration information from command line
#

   our %config_values;  # most configuration values

   $config_values{datafile} = $datafiledefault;
   $config_values{socket} = $socketdefault;

#
# Main Data
#

   my %datavalues;

# # # # # # # # # #
#   Subroutines
# # # # # # # # # #

# # # # # # # # # #
# process_request($querystring, $using_cgi)
#
# Responds to browser request and does all the work
# Returns $response, a string with the HTML response
# The $querystring is the raw query from the browser.
# Unless $using_cgi is present and true, a "security"
# parameter is used in each request to guard against
# URLs on web sites that guess we are running locally.
#

sub process_request {

   my ($querystring, $using_cgi) = @_;
   my $response;

   # Remember when we started

   my $start_clock_time = scalar localtime;
   my $start_program_time = times();

   # Get GMT time string

   my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime;
   my $dayname = qw(Sun Mon Tue Wed Thu Fri Sat)[$wday];
   my $monname = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec)[$mon];
   my $dtstring = sprintf("%s, %02d %s %04d %02d:%02d:%02d GMT", $dayname, $mday, $monname, $year+1900,
      $hour, $min, $sec);

   # Output start of HTML, including style information

   $response .= <<"EOF";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
<html>
<head>
<meta HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
<title>$programname</title>
<style type="text/css"><!--

$STYLE_STRING

-->
</style>
<script>
<!--
var setf = function() {1;}
// -->
</script>
</head>

<body onload="setf()">
EOF

   # Get CGI object to get parameters:

   $querystring ||= ""; # make sure has something

   my $q = new CGI($querystring);
   my %params = $q->Vars;

   if ($params{newtab} eq 'Quit') {
      return;
   }

   # No parameters starts on "Setup" tab

   if ($querystring eq "") {
      $params{newtab} = "Setup";
      }

   #
   # Get current tab and switch if necessary
   #

   my $currenttab = $params{newtab} || $params{currenttab};

   #
   # Display tabs and other top stuff
   #

   $response .= <<"EOF";
<b>$programname</b>
<br><br>
<form name="f0" action="" method="POST">
<input type="hidden" name="currenttab" value="$currenttab">
<table cellpadding="0" cellspacing="0" width="600">
<tr>
EOF

   foreach my $tab (@tablist) {
      if ($currenttab eq $tab) {
         $response .= <<"EOF";
<td class="tab1">&nbsp;</td>
<td class="tabselected">$tab</td>
EOF
         }
      else {
         $response .= <<"EOF";
<td class="tab1">&nbsp;</td>
<td class="tab"><input type="submit" name="newtab" value="$tab"></td>
EOF
         }
      }

   $response .= <<"EOF";
<td class="tab1" width="100%">&nbsp;</td>
</tr>
<tr>
<td class="tab2left" width="1">&nbsp;</td>
EOF

   my $ncols = @tablist;
   $ncols = $ncols * 2 + 1;
   for (my $i=0; $i<$ncols-2; $i++) {
      $response .= <<"EOF";
<td class="tab2">&nbsp;</td>
EOF
      }

   $response .= <<"EOF";
<td class="tab2right">&nbsp;</td>
</tr>
</table>
EOF

   #
   # Read in datafile
   #

   my %datavalues;
   open (DATAFILEIN, $config_values{datafile});
   while (<DATAFILEIN>) {
      chomp;
      s/\r//g; # remove CRs in case moved data from Windows
      s/^\x{EF}\x{BB}\x{BF}//; # remove UTF-8 Byte Order Mark if present
      my ($valname, $val) = split(/=/, $_, 2);
      $datavalues{$valname} = $val;

      # retrieve CR and LF's

      my %tc = ("\\" => "\\", "n" => "\n", "r" => "\r");
      $datavalues{$valname} =~ s/\\(\\|n|r)/$tc{$1}/eg;

      }
   close DATAFILEIN;

   my $changed;   # if non-zero, then a change was posted

   #
   # *** Do updates
   #

   if ($params{saveseededit}) {
      foreach my $v (qw(npeople nitems seednyse seedcpi seedshire seedchange)) {
         $datavalues{$v} = $params{"edit$v"};
         $datavalues{$v} =~ s/[^0-9\.]//g;
         }
      $changed++;
      }

   if ($params{resetdrawing}) { # reset items (including special if it's there)
      for (my $i=1; $i <= $datavalues{nitems} + 1; $i++){
         delete $datavalues{"itemwho$i"};
         }
      $changed++;
      }

   foreach my $p (keys %params) {  # go through all the parameters

      # Draw an item

      if ($p =~ /^drawitem(\d+)/) {

         my @people;
         delete $datavalues{"itemwho$1"}; # can draw more than once -- just forget last one
         for (my $i=1; $i <= $datavalues{nitems}; $i++){
            $people[$datavalues{"itemwho$i"}]++ if $datavalues{"itemwho$i"};
            }

         for (my $i=1; $i <= $datavalues{npeople}*100; $i++) { # if too few people or too many tries, give up
            my $rn = int(rand $datavalues{npeople}) + 1;
            next if $people[$rn]; # only one each
            $datavalues{"itemwho$1"} = $rn;
            last;
            }

         $changed++;
         }

      # Set item description

      if ($p =~ /^itemedit(\d+)/) {
         $datavalues{"itemname$1"} = $params{$p};;
         $changed++;
         }

      if ($p =~ m/edititemspecial(name|award)/) {
         $datavalues{"itemspecial$1"} = $params{$p};
         $changed++;
         }
      }

   #
   # *** Write out data file if changed
   #

   if ($changed) {
      open (DATAFILEOUT, "> $config_values{datafile}");
      foreach my $vn (sort keys %datavalues) {

         # escape CR and LF

         my $val = $datavalues{$vn};
         $val =~ s/\\/\\\\/g;
         $val =~ s/\n/\\n/g;
         $val =~ s/\r/\\r/g;

         print DATAFILEOUT "$vn=$val\n"; 

         }
      close DATAFILEOUT;
      }

   #
   # *** Two forms to help with making Enter
   # *** "press" the logical button)
   #

   $response .= <<"EOF";
</form>
<table cellpadding="0" cellspacing="0" width="600">
<tr>
<td class="ttbody">
EOF

   #
   # ******
   # *
   # * Output appropriate material for the tab being displayed
   # *
   # ******
   #

   my %htmlvalue;  # hash to hold values as they will be inserted into the HTML

   #
   # Setup
   #

   if ($currenttab eq "Setup") {

      $response .= <<"EOF";
<div class="pagetitle">SETUP RANDOM NUMBER GENERATOR:</div>
<form name="f1" action="" method="POST">
<input type="hidden" name="currenttab" value="$currenttab">
<br>
EOF

      if ($params{editseed}) {

         $response .= <<"EOF";
<div class="sectiondark">
<div class="pagetitle">LIMITS:</div>
<div class="title">Number of people to choose from:</div>
<input name="editnpeople" type="text" size="60" value="$datavalues{npeople}">
<div class="desc">
This should be a number, no letters or punctuation.<br>Example: 214
</div>
<div class="title">Number of items to give away:</div>
<input name="editnitems" type="text" size="60" value="$datavalues{nitems}">
<div class="desc">
This should be a number, no letters or punctuation.<br>Example: 8
</div>
</div>
<br>
<div class="sectiondark">
<div class="pagetitle">INITIALIZERS:</div>
<div class="title">Number of Shares Last Traded on NYSE:</div>
<input name="editseednyse" type="text" size="60" value="$datavalues{seednyse}">
<div class="desc">
The number of shares last traded on the New York Stock Exchange.
</div>
<div class="title">Most Recent Bureau of Labor Statistics CPI-U:</div>
<input name="editseedcpi" type="text" size="60" value="$datavalues{seedcpi}">
<div class="desc">
The most recently reported value of the Consumer Price Index for All Urban
Consumers, before seasonal adjustment.
</div>
<div class="title">Latest Bid on a Shire on eBay:</div>
<input name="editseedshire" type="text" size="60" value="$datavalues{seedshire}">
<div class="desc">
The latest bid (in pennies) for a "shire" on eBay.
</div>
<div class="title">Spare Change in Professor's Pocket:</div>
<input name="editseedchange" type="text" size="60" value="$datavalues{seedchange}">
<div class="desc">
The amount (in pennies) in the pocket of a chosen professor in attendence.
</div>
</div>
<br>
<input name="saveseededit" type="submit" value="Save">
<input name="cancelseededit" type="submit" value="Cancel">
</form>
<script>
<!--
var setf = function() {document.f1.editnpeople.focus();}
// -->
</script>
EOF
         }

      else {
         if (!$datavalues{npeople} || !$datavalues{nitems}) {
            $response .= <<"EOF";
<span class="warning">
Click the Edit button below to set values for the number of people,
number of items to give away,
and the random number initializers.
Save those, and then set the item descriptions on the "Items" tab.
Finally, draw for the items using the "Drawing" tab.
</span>
<br><br>
EOF
            }
         $response .= <<"EOF";
<div class="itembuttons"><input class="small" name="editseed" type="submit" value="Edit"></div>
<div class="itemheader">Limits:</div>
<div class="itemmisc">
<table cellpadding="0" cellspacing="0">
<tr><td width="150"><b>Number of people:</b></td><td>$datavalues{npeople}</td></tr>
<tr><td width="150"><b>Number of items:</b></td><td>$datavalues{nitems}</td></tr>
</table>
</div>
<div class="itemheader">Initializers:</div>
<div class="itemmisc">
<table cellpadding="0" cellspacing="0">
<tr><td width="150"><b>NYSE Shares Traded:</b></td><td>$datavalues{seednyse}</td></tr>
<tr><td width="150"><b>CPI-U:</b></td><td>$datavalues{seedcpi}</td></tr>
<tr><td width="150"><b>Shire bid on eBay:</b></td><td>$datavalues{seedshire}</td></tr>
<tr><td width="150"><b>Spare change:</b></td><td>$datavalues{seedchange}</td></tr>
</table>
</div>
</form>
<script>
<!--
var setf = function() {document.f1.editseed.focus();}
// -->
</script>
<br>
EOF
         }

      }

   #
   # Test
   #

   if ($currenttab eq "Test") {

      $datavalues{npeople} = 2 unless $datavalues{npeople}; # watch out for not set!

      $response .= <<"EOF";
<form name="f1" action="" method="POST">
<input type="hidden" name="currenttab" value="$currenttab">
<div class="pagetitle">TEST:</div>
<br>
EOF

      if ($params{runtest1}) {

         #
         # Initialize random number generator
         #

         my ($snyse, $scpi, $sshire, $schange) = ($datavalues{seednyse}, $datavalues{seedcpi}, $datavalues{seedshire}, $datavalues{seedchange});
         $snyse =~ s/\D//g;
         $scpi =~ s/\D//g;
         $sshire =~ s/\D//g;
         $schange =~ s/\D//g;
         srand ($snyse + $scpi + $sshire + $schange);
         rand; # run once so a low number seed won't always choose number 1

         my @people;
         my $npeople = $datavalues{npeople};
         for (my $i=1; $i<=$npeople*$params{testn}; $i++) {
            my $rn = int(rand $npeople) + 1;
            $people[$rn]++;
            }

         my %distribution;
         my $average;
         my $high;
         my $low = $npeople * $params{testn};
         for (my $i=1; $i<=$npeople; $i++) {
            $distribution{sprintf("%2d", $people[$i])}++;
            $average += $people[$i];
            $low = $people[$i] if $people[$i] < $low;
            $high = $people[$i] if $people[$i] > $high;
            }
         $average /= $npeople;
         my $stddev;
         for (my $i=1; $i<=$npeople; $i++) {
            $stddev += ($people[$i] - $average)**2;
            }
         $stddev = sqrt($stddev / $npeople);

         $response .= <<"EOF";
The test was run $params{testn} times on $datavalues{npeople} people.
Here are the number of times each was chosen. Ideally, in this test
you would like all to be chosen as close to $params{testn} times as possible
with the standard deviation to be very small compared to the average.
<br><br>
<div class="sectiondark">
EOF

         $response .= sprintf("<b>Average=%d</b><br><span class=\"smallprompt\">", $average);

         $response .= sprintf("Highest=%d (%0.1f%% more than average), Lowest=%d (%0.1f%% smaller than average), Standard Deviation=%0.2f (%0.1f%% of average)</span>",
                      $high, 100*($high-$average)/$average, $low, 100*($average-$low)/$average, $stddev, 100*$stddev/$average);

         $response .= <<"EOF";
<br></div><br>
<input name="donetest" type="submit" value="Run Again">
<script>
<!--
var setf = function() {document.f1.donetest.focus();}
// -->
</script>
<br>
<br>For each person number, here are the number of times it was chosen:
<br>
EOF
         for (my $i=1; $i<=$npeople; $i++) {
            $response .= sprintf("<br>%03d: %d", $i, $people[$i]);
            }
         $response .= <<"EOF";
<br>
EOF
         }

      elsif ($params{runtest2}) {

         my @people;
         my $npeople = $datavalues{npeople};

         # Get initial random number values

         my ($snyse, $scpi, $sshire, $schange) = ($datavalues{seednyse}, $datavalues{seedcpi}, $datavalues{seedshire}, $datavalues{seedchange});
         $snyse =~ s/\D//g;
         $scpi =~ s/\D//g;
         $sshire =~ s/\D//g;
         $schange =~ s/\D//g;

         for (my $i=1; $i<=$params{testn}; $i++) { # run test this many times
            srand ($snyse + $scpi + $sshire + $schange + $i - 1); # initialize random number generator and add increment
            rand; # run once so a low number seed won't always choose number 1

            for (my $j=1; $j<=$datavalues{nitems}; $j++) { # choose this many items (don't worry about duplicates for now)
               my $rn = int(rand $npeople) + 1;
               $people[$rn]++;
               }
            }

         my $different;
         my @decile;
         for (my $i=1; $i<=$npeople; $i++) {
            $different++ if $people[$i];
            $decile[10*($i-1)/$npeople] += $people[$i];
            }
         my $hoped = $datavalues{nitems} * $params{testn};
         $hoped = $hoped < $npeople ? $hoped : $npeople;
         my $off = sprintf("%0.1f%%", 100*$different/$hoped);
         my $ideal = int($datavalues{nitems}*$params{testn}/$npeople + 0.75);
         my $ideal1 = $ideal || 1;

         $response .= <<"EOF";
<div class="sectiondark">
The test was run $params{testn} times,
each time choosing $datavalues{nitems} people out of $datavalues{npeople}.
There were a total of $different different people chosen,
which is $off of the $hoped you would have hoped for.<br><br>
Here are the number of times each was chosen. Ideally, in this test
you would like all that are chosen to be chosen about $ideal1 time(s) and
the particular ones spread out throughout the range.
EOF

         $response .= <<"EOF";
<br></div><br>
<input name="donetest" type="submit" value="Test Again">
<script>
<!--
var setf = function() {document.f1.donetest.focus();}
// -->
</script>
<br>
EOF

         $response .= <<"EOF";
<br>For each decile (10%) range chosen, here are the number of times the average member in it was chosen
(you would expect around $ideal):
<br>
EOF
         for (my $i=1; $i<=10; $i++) {
            $response .= sprintf("<br>%02d: %0.1f", $i, 10*$decile[$i-1]/$npeople);
            }

         $response .= <<"EOF";
<br>
<br>For each person number chosen, here are the number of times it was chosen:
<br>
EOF
         for (my $i=1; $i<=$npeople; $i++) {
            $response .= sprintf("<br>%03d: %d", $i, $people[$i]) if $people[$i];
            }
         $response .= <<"EOF";
<br>
EOF
         }

      else {

         $response .= <<"EOF";
<div class="sectiondark">
These tests let you see characteristics of the random numbers
generated by this program using the initializer values chosen.
It produces repeatable results, that is, the sequence of numbers
will always be the same given the same initializer values.
<br><br>
The first test works by initializing the sequence, and then
choosing $datavalues{npeople} numbers from 1 to $datavalues{npeople}, remembering how
many times each number has been chosen.
The test may be run multiple times, continuing in sequence.
<br><br>
The second test works by initializing the sequence, and then
choosing $datavalues{nitems} numbers from 1 to $datavalues{npeople}, remembering
which numbers are chosen. It then reinitializes the sequence with
initializers that are one more than before, and runs again.
Each chosen value is then listed with the number of times chosen.
This is a better test of "fairness" of this general program than the first test.
</div>
<br>
<b>Number of times to run: </b>
<input name="testn" type="radio" value="1"><span class="smallprompt">x1</span>
<input name="testn" type="radio" value="5"><span class="smallprompt">x5</span>
<input name="testn" type="radio" value="10"><span class="smallprompt">x10</span>
<input name="testn" type="radio" value="50"><span class="smallprompt">x50</span>
<input name="testn" type="radio" value="100" CHECKED><span class="smallprompt">x100</span>
<input name="testn" type="radio" value="1000"><span class="smallprompt">x1000</span>
<input name="testn" type="radio" value="10000"><span class="smallprompt">x10000</span>
<br><span class="smallprompt">(Testing on $datavalues{npeople}.)</span>
<br><br>
<input name="runtest1" type="submit" value="Run Test 1">
<input name="runtest2" type="submit" value="Run Test 2">
<script>
<!--
var setf = function() {document.f1.runtest1.focus();}
// -->
</script>
EOF
          }

       $response .= <<"EOF";
</form>
EOF

      }

   #
   # Items
   #

   if ($currenttab eq "Items") {

      $response .= <<"EOF";
<div class="pagetitle">DESCRIPTIONS OF ITEMS:</div>
<form name="f1" action="" method="POST">
<input type="hidden" name="currenttab" value="$currenttab">
<br><br>
EOF

      if ($params{edititems}) {

         $response .= <<"EOF";
<div class="sectiondark">
<div class="pagetitle">ITEM DESCRIPTIONS:</div>
EOF

         for (my $i=1; $i<=$datavalues{nitems}; $i++) {
            my $itemnamesc = special_chars($datavalues{"itemname$i"});
            $response .= <<"EOF";
<div class="title">Item $i description:</div>
<input name="itemedit$i" type="text" size="60" value="$itemnamesc">
<div class="desc">
A short description of the item.
</div>
EOF
            }

         my $itemnamesc = special_chars($datavalues{itemspecialname});
         my $itemawardsc = special_chars($datavalues{itemspecialaward});
         $response .= <<"EOF";
<div class="title">"Special Item" description:</div>
<input name="edititemspecialname" type="text" size="60" value="$itemnamesc">
<div class="desc">
A short description of an item to be added to the end for "surprise" fixed giveaways.
When they are "drawn", the "award" text below is displayed.
If this is blank, then there is no special item.
</div>
<div class="title">"Special Item" award text:</div>
<textarea name="edititemspecialaward" rows="3" cols="60" wrap="virtual">$itemawardsc</textarea>
<div class="desc">
The text to display when the special item is drawn instead of a number.
</div>
</div>
<br>

<input name="saveitemedit" type="submit" value="Save">
<input name="cancelitemedit" type="submit" value="Cancel">
</form>
<script>
<!--
var setf = function() {document.f1.itemedit1.focus();}
// -->
</script>
EOF
         }

      else {
         $response .= <<"EOF";
<div class="itembuttons"><input class="small" name="edititems" type="submit" value="Edit"></div>
<div class="itemheader">Item number and description:</div>
<div class="itemmisc">
<table cellpadding="0" cellspacing="0">
EOF

         for (my $i=1; $i<=$datavalues{nitems}; $i++) {
            my $itemnamesc = special_chars($datavalues{"itemname$i"});
            $response .= <<"EOF";
<tr><td width="50"><b>Item $i</b></td><td>$itemnamesc</td></tr>
EOF
            }

         my $itemnamesc = special_chars($datavalues{itemspecialname});
         if ($itemnamesc) { # display special if there is one
            my $i = $datavalues{nitems} + 1;
            $response .= <<"EOF";
<tr><td width="50"><b>Item $i</b></td><td>$itemnamesc</td></tr>
EOF
            }

         $response .= <<"EOF";
</table>
</div>
</form>
<script>
<!--
var setf = function() {document.f1.edititems.focus();}
// -->
</script>
<br>
EOF
         }
      }

   #
   # Drawing
   #

   if ($currenttab eq "Drawing") {

      $datavalues{npeople} = 2 unless $datavalues{npeople}; # watch out for not set!

      $response .= <<"EOF";
<form name="f1" action="" method="POST">
<input type="hidden" name="currenttab" value="$currenttab">
<div class="pagetitle">DRAWING:</div>
<br>
EOF

      if ($params{initializerand}) {

         #
         # Initialize random number generator
         #

         my ($snyse, $scpi, $sshire, $schange) = ($datavalues{seednyse}, $datavalues{seedcpi}, $datavalues{seedshire}, $datavalues{seedchange});
         $snyse =~ s/\D//g;
         $scpi =~ s/\D//g;
         $sshire =~ s/\D//g;
         $schange =~ s/\D//g;
         srand ($snyse + $scpi + $sshire + $schange);
         rand; # run once so a low number seed won't always choose number 1

         $response .= <<"EOF";
<div class="sectiondark">
<div class="pagetitle">Random number generator initialized using:</div><br>
<div class="itemdraw">
<table cellpadding="0" cellspacing="0">
<tr><td width="350"><b>NYSE Shares Traded:</b></td><td>$datavalues{seednyse}</td></tr>
<tr><td width="350"><b>CPI-U:</b></td><td>$datavalues{seedcpi}</td></tr>
<tr><td width="350"><b>Shire bid on eBay:</b></td><td>$datavalues{seedshire}</td></tr>
<tr><td width="350"><b>Spare change:</b></td><td>$datavalues{seedchange}</td></tr>
</table>
</div>
<br></div><br>
<input name="resetdrawing" type="submit" value="Continue">
<input name="initialized" type="hidden" value="yes">
<script>
<!--
var setf = function() {document.f1.resetdrawing.focus();}
// -->
</script>
<br>
EOF
         }

      elsif (!$params{initialized}) {

         $response .= <<"EOF";
<div class="sectiondark">
<b>Initialize the random number generator and reset the drawing by pressing this button:</b><br><br>
<input name="initializerand" type="submit" value="Initialize">
<span class="smallprompt">(Do this once.)</span>
<br><br><span class="smallprompt">Only one item per person.
(There are $datavalues{nitems} items and $datavalues{npeople} people.)</span>
</div>
<br>
EOF
         }

      else {

         $response .= <<"EOF";
<input name="initialized" type="hidden" value="yes">
<table cellspacing="0" cellpadding="0" width="100%">
<tr><td align="right"><input name="initializerand" type="submit" class="small" value="Reinitialize"></td></tr></table>
<br>
EOF

         my $nextbutton;
         for (my $inum=1; $inum <= $datavalues{nitems}; $inum++) {
            my $itemnamesc = special_chars($datavalues{"itemname$inum"});
            if ($datavalues{"itemwho$inum"}) {
               $response .= <<"EOF";
<div class="sectiondark">
<table cellspacing="0" cellpadding="0" width="100%">
<tr>
<td>
<table cellspacing="0" cellpadding="0">
<tr><td class="centered"><span style="font-size:32pt;font-weight:bold;">$datavalues{"itemwho$inum"}&nbsp;</span></td>
<td class="centered"><span style="font-size:14pt;font-weight:bold;">$itemnamesc</span> (item $inum)</td></tr></table>
</td>
<td align="right">
&nbsp;<input name="drawitem$inum" type="submit" class="small" value="Redraw">
</td></tr></table>
</div><br>
EOF
               }
            else {
               $response .= <<"EOF";
<table cellspacing="0" cellpadding="0">
<tr>
<td width="40">&nbsp;</td>
<td valign="top"><input name="drawitem$inum" type="submit" value="Draw">&nbsp;</td>
<td>$itemnamesc (item $inum)</td></tr></table>
<br>
EOF
            $nextbutton = $inum unless $nextbutton;
               }
            }

         if ($datavalues{itemspecialname}) { # If there is a special item
            my $inum = $datavalues{nitems} + 1;
            if ($datavalues{"itemwho$inum"}) {
               my $itemnamesc = special_chars($datavalues{itemspecialaward});
               $itemnamesc =~ s/\n/<br>\n/g;
               $response .= <<"EOF";
<div class="sectiondark">
<table cellspacing="0" cellpadding="0" width="100%">
<tr>
<td>
<b>***</b><br>
<span style="font-size:18pt;font-weight:bold;">$itemnamesc</span>
<br></b>***</b>
<br>(item $inum)
</td>
<td>&nbsp;</td></tr>
<tr><td>&nbsp;</td>
<td align="right">
&nbsp;<input name="drawitem$inum" type="submit" class="small" value="Redraw">
</td></tr></table>
</div><br>
EOF
               $nextbutton = $inum unless $nextbutton;
               }
            else {
               my $itemnamesc = special_chars($datavalues{itemspecialname});
               $response .= <<"EOF";
<table cellspacing="0" cellpadding="0">
<tr>
<td width="40">&nbsp;</td>
<td><input name="drawitem$inum" type="submit" value="Draw">&nbsp;</td>
<td>$itemnamesc (item $inum)</td></tr></table>
<br>
EOF
               $nextbutton = $inum unless $nextbutton;
               }
            }

         $response .= <<"EOF";
<input name="drawitem" type="hidden" value="">
<script>
<!--
var setf = function() {document.f1.drawitem$nextbutton.focus();}
// -->
</script>
EOF
          }

       $response .= <<"EOF";
</form>
EOF

      }

   #
   # Show License
   #

   if ($currenttab eq "Show License") {

         use HBS79License;

         $response .= <<"EOF";
<div class="sectiondark">
$sgilicensetext
</div>
<br>
<div class="sectiondark">
$gpllicensetext
</div>
EOF

      }

   # *** Output common stuff

   $response .= <<"EOF";
</td>
</tr>
EOF

   # Finish -- output closing stuff

   my $end_time = times();
   my $time_string = sprintf ("Runtime %.2f seconds at $start_clock_time",
                              $end_time - $start_program_time);
   #
   # *********
   #
   # If you modify the program, add your copyright and modification notices here.
   # Do not remove previous ones.
   #
   # *********
   #

   $response .= <<"EOF";
<tr><td>
<div class="footer">
$time_string<br><br>
(c) Copyright 2004 Software Garden, Inc.
<br>All Rights Reserved.
EOF

   $response .= <<"EOF";
<br><br>
<form name="fsl" action="" method="POST">
<input type="hidden" name="currenttab" value="$currenttab">
<input style="font-size:xx-small;" type="submit" name="newtab" value="Show License">
</form>
</div>
</td></tr>
EOF

   $response .= <<"EOF";
</table>
</body>
</html>
EOF

   return $response;

   }

# # # # # # # # # #
# html_escape(\%instrings, \%outstrings, @stringkeys)
#
# Accesses @stringkeys elements of %instrings,
# does HTML escaping for &, <, >, "
# and then adds the results to %outstrings
# 

sub html_escape {
   my $instrings = shift @_;
   my $outstrings = shift @_;

   foreach my $skey (@_) {
      my $string = $$instrings{$skey};
      $string =~ s/&/&amp;/g;
      $string =~ s/</&lt;/g;
      $string =~ s/>/&gt;/g;
      $string =~ s/"/&quot;/g;

      $$outstrings{$skey} = $string;
      }

   return;
}

# # # # # # # # # #
# special_chars($string)
#
# Returns $estring where &, <, >, " are HTML escaped
# 

sub special_chars {
   my $string = shift @_;

   $string =~ s/&/&amp;/g;
   $string =~ s/</&lt;/g;
   $string =~ s/>/&gt;/g;
   $string =~ s/"/&quot;/g;

   return $string;
}


# # # # # # # # # #
# $outstring = expand_template($template, \%valuestrings)
#
# Copies $templatestring to result, and for
# {{key}} accesses %valuestrings.
# 

sub expand_template {
   my $template = shift @_;
   my $valuestrings = shift @_;
   my $outstring = $template;

   $outstring =~ s/{{(\w+?)}}/$$valuestrings{$1}/eg;

   return $outstring;
}


1; # For use

__END__

=head1 NAME

ListGarden.pm

=head1 VERSION

This is ListGarden.pm v1.0.

=head1 AUTHOR

Dan Bricklin, Software Garden, Inc.

=head1 COPYRIGHT

(c) Copyright 2004 Software Garden, Inc.
All Rights Reserved.

See Software License in the program file.

=cut

#
# HISTORY
#
# Version 1.00
# $Date: 2004/07/08 14:01:30 $
# $Revision: 1.8 $
#
# Version 1.02 20 Sep 2004 17:01 EDT
#   Added Channel Additional XML (channeladdlxml) feature
#
# Version 1.02 20 Sep 2004 15:18 EDT
#   Added File::Glob to handle pathnames with spaces
#
# Version 1.02 8 Jul 2004 16:03 EDT
#   Moved <script> tags to after target defined for cleanliness
#
# Version 1.01 7 Jul 2004 11:50 EDT
#   Multiple forms and button reordering to fix "text and Enter" problem
#   Better browser compatibility with DOCTYPE and CSS tweaks
#   Clean up HTML in various places to look better
#   UTF-8 BOM handling
#   When no feeds, focus is put in Create text field
#   Handle text in numeric fields better
#
# Version 1.00 28 Jun 2004 22:35 EDT
#   Dan Bricklin, Software Garden, Inc. (http://www.softwaregarden.com/)
#   -Intitial version
#
#
# TODO:
#
#

=begin license

SOFTWARE LICENSE

This software and documentation is
Copyright (c) 2004 Software Garden, Inc.
All rights reserved. 

1. The source code of this program is made available as free software;
you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version. 

2. This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details. You should have received a
copy of the GNU General Public License along with this program; if not,
write to the Free Software Foundation, Inc., 59 Temple Place - Suite
330, Boston, MA  02111-1307, USA.

3. If the GNU General Public License is restrictive in a way that does
not meet your needs, contact the copyright holder (Software Garden,
Inc.) to inquire about the availability of other licenses, such as
traditional commercial licenses. 

4. Redistributions in source code must retain the above copyright
notice, this list of conditions, and the following disclaimer. 

5. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions, and the following disclaimer in the
documentation and/or other materials provided with the distribution. 

6. The end-user documentation included with the redistribution, if any,
must include the following acknowledgment:

   This product includes software developed by Software Garden, Inc.
   (www.softwaregarden.com). 

Alternately, this acknowledgment may appear in the software itself, if
and wherever such third-party acknowledgments normally appear (such as
this notice). 

7. The right to distribute this software or to use it for any purpose
does not give you the right to use Servicemarks or Trademarks of
Software Garden, Inc., including Garden, Software Garden, and
ListGarden. 

8. If any files are modified, you must cause the modified files to
carry prominent notices stating that you changed the files and the date
and nature of any change. 

9. Modified versions that are redistributed must not remove rights or
licensing notices present in the product, nor cause them not to be
displayed when the software is executed. 

Exception for Executable Bundle 

In some cases this program is distributed together with programs and
libraries of ActiveState Corporation as a single executable file (an
"Executable Bundle") produced using ActiveState Corporation's "Perl Dev
Kit" PerlTray program ("PDK PerlTray"). This free software license does
not apply to those programs and libraries of ActiveState Corporation
that are part of the Executable Bundle. You only have a license to use
those programs and libraries of ActiveState Corporation for runtime
purposes in order to execute this software of Software Garden, Inc. In
order to create and distribute similar executable files from modified
source files, you will need to license your own copy of PDK PerlTray. 

As a specific exception for this product to the terms and conditions of
the GNU General Public License version 2, you are free to distribute
this software (modified or unmodified) in an Executable Bundle created
with PDK PerlTray as long as you adhere to the GNU General Public
License in all respects for all software components except for those of
PDK PerlTray added by that program when used to create the Executable
Bundle. 

Disclaimer 

THIS SOFTWARE IS PROVIDED BY SOFTWARE GARDEN, INC., "AS IS" AND ANY
EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
WARRANTIES OF INFRINGEMENT AND THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL SOFTWARE GARDEN, INC. NOR ITS EMPLOYEES AND OFFICERS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE DISTRIBUTION OR USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 

Software Garden, Inc.
PO Box 610369
Newton Highlands, MA 02461 USA
www.softwaregarden.com

License version: 1.0/2004-06-24 

=end

=cut
